@@ -26,7 +26,7 @@ def inspect(manager):
 
 
 class test_Inspect:
-    """Integration tests fo app.control.inspect() API"""
+    """Integration tests to app.control.inspect() API"""
 
     @flaky
     def test_ping(self, inspect):
@@ -51,7 +51,7 @@ class test_Inspect:
         # TODO: We can check also the exact values of the registered methods
         ret = inspect.registered()
         assert len(ret) == 1
-        len(ret[NODENAME]) > 0
+        assert len(ret[NODENAME]) > 0
         for task_name in ret[NODENAME]:
             assert isinstance(task_name, str)
 
