@@ -52,7 +52,7 @@ class test_flatten_reply:
 
 class test_inspect:
 
-    def setup(self):
+    def setup_method(self):
         self.app.control.broadcast = Mock(name='broadcast')
         self.app.control.broadcast.return_value = {}
         self.inspect = self.app.control.inspect()
@@ -207,7 +207,7 @@ class test_inspect:
 
 class test_Control_broadcast:
 
-    def setup(self):
+    def setup_method(self):
         self.app.control.mailbox = Mock(name='mailbox')
 
     def test_broadcast(self):
@@ -231,7 +231,7 @@ class test_Control_broadcast:
 
 class test_Control:
 
-    def setup(self):
+    def setup_method(self):
         self.app.control.broadcast = Mock(name='broadcast')
         self.app.control.broadcast.return_value = {}
 
@@ -424,6 +424,16 @@ class test_Control:
             terminate=False,
         )
 
+    def test_revoke_by_stamped_headers(self):
+        self.app.control.revoke_by_stamped_headers({'foo': 'bar'})
+        self.assert_control_called_with_args(
+            'revoke_by_stamped_headers',
+            destination=None,
+            headers={'foo': 'bar'},
+            signal=control.TERM_SIGNAME,
+            terminate=False,
+        )
+
     def test_revoke__with_options(self):
         self.app.control.revoke(
             'foozbaaz',
@@ -441,6 +451,23 @@ class test_Control:
             _options={'limit': 404},
         )
 
+    def test_revoke_by_stamped_headers__with_options(self):
+        self.app.control.revoke_by_stamped_headers(
+            {'foo': 'bar'},
+            destination='a@q.com',
+            terminate=True,
+            signal='KILL',
+            limit=404,
+        )
+        self.assert_control_called_with_args(
+            'revoke_by_stamped_headers',
+            destination='a@q.com',
+            headers={'foo': 'bar'},
+            signal='KILL',
+            terminate=True,
+            _options={'limit': 404},
+        )
+
     def test_election(self):
         self.app.control.election('some_id', 'topic', 'action')
         self.assert_control_called_with_args(
@@ -499,6 +526,14 @@ class test_Control:
             connection=None, reply=False, signal=None,
             terminate=False, timeout=None)
 
+    def test_revoke_by_stamped_headers_from_result(self):
+        self.app.control.revoke_by_stamped_headers = Mock(name='revoke_by_stamped_headers')
+        self.app.AsyncResult('foozbazzbar').revoke_by_stamped_headers({'foo': 'bar'})
+        self.app.control.revoke_by_stamped_headers.assert_called_with(
+            {'foo': 'bar'},
+            connection=None, reply=False, signal=None,
+            terminate=False, timeout=None)
+
     def test_revoke_from_resultset(self):
         self.app.control.revoke = Mock(name='revoke')
         uuids = [uuid() for _ in range(10)]
