@@ -12,7 +12,7 @@ pytest.importorskip('boto3')
 
 
 class test_DynamoDBBackend:
-    def setup(self):
+    def setup_method(self):
         self._static_timestamp = Decimal(1483425566.52)
         self.app.conf.result_backend = 'dynamodb://'
 
@@ -121,23 +121,6 @@ class test_DynamoDBBackend:
         mock_set_table_ttl.assert_called_once()
 
     def test_get_or_create_table_not_exists(self):
-        self.backend._client = MagicMock()
-        mock_create_table = self.backend._client.create_table = MagicMock()
-        mock_describe_table = self.backend._client.describe_table = \
-            MagicMock()
-
-        mock_describe_table.return_value = {
-            'Table': {
-                'TableStatus': 'ACTIVE'
-            }
-        }
-
-        self.backend._get_or_create_table()
-        mock_create_table.assert_called_once_with(
-            **self.backend._get_table_schema()
-        )
-
-    def test_get_or_create_table_already_exists(self):
         from botocore.exceptions import ClientError
 
         self.backend._client = MagicMock()
@@ -145,15 +128,27 @@ class test_DynamoDBBackend:
         client_error = ClientError(
             {
                 'Error': {
-                    'Code': 'ResourceInUseException',
-                    'Message': 'Table already exists: {}'.format(
-                        self.backend.table_name
-                    )
+                    'Code': 'ResourceNotFoundException'
                 }
             },
-            'CreateTable'
+            'DescribeTable'
         )
-        mock_create_table.side_effect = client_error
+        mock_describe_table = self.backend._client.describe_table = \
+            MagicMock()
+        mock_describe_table.side_effect = client_error
+        self.backend._wait_for_table_status = MagicMock()
+
+        self.backend._get_or_create_table()
+        mock_describe_table.assert_called_once_with(
+            TableName=self.backend.table_name
+        )
+        mock_create_table.assert_called_once_with(
+            **self.backend._get_table_schema()
+        )
+
+    def test_get_or_create_table_already_exists(self):
+        self.backend._client = MagicMock()
+        mock_create_table = self.backend._client.create_table = MagicMock()
         mock_describe_table = self.backend._client.describe_table = \
             MagicMock()
 
@@ -167,6 +162,7 @@ class test_DynamoDBBackend:
         mock_describe_table.assert_called_once_with(
             TableName=self.backend.table_name
         )
+        mock_create_table.assert_not_called()
 
     def test_wait_for_table_status(self):
         self.backend._client = MagicMock()
