@@ -66,9 +66,7 @@ class test_process_initializer:
     def test_process_initializer(self, _signals, set_mp_process_title, restore_logging):
         from celery import signals
         from celery._state import _tls
-        from celery.concurrency.prefork import (WORKER_SIGIGNORE,
-                                                WORKER_SIGRESET,
-                                                process_initializer)
+        from celery.concurrency.prefork import WORKER_SIGIGNORE, WORKER_SIGRESET, process_initializer
         on_worker_process_init = Mock()
         signals.worker_process_init.connect(on_worker_process_init)
 
@@ -196,13 +194,14 @@ class ExeMockTaskPool(mp.TaskPool):
 @t.skip.if_win32
 class test_AsynPool:
 
-    def setup(self):
+    def setup_method(self):
         pytest.importorskip('multiprocessing')
 
     def test_gen_not_started(self):
 
         def gen():
             yield 1
+            assert not asynpool.gen_not_started(g)
             yield 2
         g = gen()
         assert asynpool.gen_not_started(g)
@@ -211,6 +210,17 @@ class test_AsynPool:
         list(g)
         assert not asynpool.gen_not_started(g)
 
+        def gen2():
+            yield 1
+            raise RuntimeError('generator error')
+        g = gen2()
+        assert asynpool.gen_not_started(g)
+        next(g)
+        assert not asynpool.gen_not_started(g)
+        with pytest.raises(RuntimeError):
+            next(g)
+        assert not asynpool.gen_not_started(g)
+
     @patch('select.select', create=True)
     def test_select(self, __select):
         ebadf = socket.error()
@@ -355,11 +365,23 @@ class test_AsynPool:
         pool.register_with_event_loop(hub)
         hub.on_tick.add.assert_called_once()
 
+    @patch('billiard.pool.Pool._create_worker_process')
+    def test_before_create_process_signal(self, create_process):
+        from celery import signals
+        on_worker_before_create_process = Mock()
+        signals.worker_before_create_process.connect(on_worker_before_create_process)
+        pool = asynpool.AsynPool(processes=1, threads=False)
+        create_process.assert_called_once_with(0)
+        on_worker_before_create_process.assert_any_call(
+            signal=signals.worker_before_create_process,
+            sender=pool,
+        )
+
 
 @t.skip.if_win32
 class test_ResultHandler:
 
-    def setup(self):
+    def setup_method(self):
         pytest.importorskip('multiprocessing')
 
     def test_process_result(self):
