@@ -1,5 +1,6 @@
 import copy
 import datetime
+import platform
 import traceback
 from contextlib import contextmanager
 from unittest.mock import Mock, call, patch
@@ -389,12 +390,17 @@ class test_AsyncResult:
 
         assert not self.app.AsyncResult(uuid()).ready()
 
+    @pytest.mark.skipif(
+        platform.python_implementation() == "PyPy",
+        reason="Mocking here doesn't play well with PyPy",
+    )
     def test_del(self):
         with patch('celery.result.AsyncResult.backend') as backend:
             result = self.app.AsyncResult(self.task1['id'])
+            result.backend = backend
             result_clone = copy.copy(result)
             del result
-            assert backend.remove_pending_result.called_once_with(
+            backend.remove_pending_result.assert_called_once_with(
                 result_clone
             )
 
@@ -967,6 +973,13 @@ class test_EagerResult:
             res_subtask_async.get()
         res_subtask_async.get(disable_sync_subtasks=False)
 
+    def test_populate_name(self):
+        res = EagerResult('x', 'x', states.SUCCESS, None, 'test_task')
+        assert res.name == 'test_task'
+
+        res = EagerResult('x', 'x', states.SUCCESS, name='test_task_named_argument')
+        assert res.name == 'test_task_named_argument'
+
 
 class test_tuples:
 
